import java.lang.Math.*;
/**
 * @author  E. Alkan 
 * @version 22.11.2007
 */
public class SortierFeld
{
    // Instanzvariablen
    private int maximum;
    private int laenge;
    private int vergleiche;
    private int zuweisungen;
    
    
    
    private int[] feld;

    /**
     * Konstruktor fr Objekte der Klasse Felder
     * Erzeugt ein Zufallsfeld mit "anzahl" Integerwerten die maximal "maximum" gross sind
     */
    public SortierFeld(int anzahl, int maximum)
    {
        //Instanzvariablen deklarieren
        this.maximum = maximum;
        this.laenge = anzahl;
        feld = new int[anzahl];
        // Instanzvariable initialisieren
        for(int i=0;i<feld.length;i++) {
            feld[i]=(int)(maximum*(Math.random()))+1;
        }
        vergleiche = 0;
        zuweisungen = 0;
    }
    
    public int getVergleiche()
    {
        return vergleiche;
    }
    
    public int getZuweisungen()
    {
        return zuweisungen;
    }
    
    public void setVergleiche(int vergleiche)
    {
        this.vergleiche = vergleiche;
    }
    
    public void setZuweisungen(int zuweisungen)
    {
        this.zuweisungen = zuweisungen;
    }
  
	public int getMaximum()
    {
        return maximum;
    }

    public int getLaenge()
    {
        return laenge;
    }

    public int[] getFeld()
    {
        return feld;
    }

    public int getFeld(int stelle)
    {
        if ((stelle>=0)&&(stelle<feld.length))
            return feld[stelle];
        else
            return -1;
    }

    public void initAufsteigendSortiert()
    {
        for(int i=0;i<feld.length;i++)
        {
            feld[i]=i;
        }
    }
    
    public void initAbsteigendSortiert()
    {
        for(int i=0;i<feld.length;i++)
        {
            feld[i]=feld.length-i-1;
        }
    }
    
    public void initZufaellig()
    {
        for(int i=0;i<feld.length;i++)
        {
            feld[i]=(int)(getMaximum()*(Math.random()))+1;
        }
    }
    
    /**
     * Das gesamte Feld ausgeben
     * (Achtung! Keine Rckgabe!)
     */
    public void gesamtesFeldAusgeben()
    {
        for(int i=0;i<feld.length;i++) {
            System.out.print(feld[i] + " ");
        }
        System.out.println();
    }

    /**
     * x-tes und y-tes Element aus dem Feld vertauschen
     */
    public void tausche(int x, int y)
    {
        if ((x>=0)&&(x<feld.length)&&(y>=0)&&(y<feld.length))
        {
            int h   = feld[x]; // Hilfsvariable fr den Dreieckstausch
            feld[x] = feld[y];
            feld[y] = h;
        }
    }


    /**
     * Insertsort
     */
    public void insertSort()
    {
        for (int i=1; i<feld.length;i++)
        {
            int hilfsElement = feld[i];
            int j = i-1;
            
            while (j>=0 && hilfsElement<feld[j])
            {
                feld[j+1] = feld[j];
                j--;
            }
            feld[j+1] = hilfsElement;;
        }
    }

    /**
     * Selectsort
     */
    public void selectSort()
    {
        for (int i = 0;i<feld.length;i++)
        {
            int minPos=i;
            for (int j = i+1;j<feld.length;j++)
            {
                if (feld[j]<feld[minPos])
                {
                    minPos = j;
                }
            }
            tausche(minPos,i);
        }
    }

    /**
     * Bubblesort
     */
    public void bubbleSort()
    {
        int n=feld.length-1;
        boolean getauscht;
        do {
            getauscht = false;
            for (int i=0; i<n;i++)
            {
                if (getFeld(i) > getFeld(i+1))
                {
                    tausche(i, i+1);
                    getauscht = true;
                }
            }
            n--;
            
        } while (getauscht);
    }

    /**
     * Mergesort
     */
    public void mergeSort(int l, int r)
    {
        int m = 0;
        if (l < r)
        {
            m = (l + r)/2;
            mergeSort(l, m);
            mergeSort(m+1,r);
            merge(l,m,r);
        }
    }
    
    public void merge(int l, int m, int r)
    {
        int[] hFeld = new int[feld.length]; // Hilfsfeld
        int i,j,k;
        
        i = l;
        j = m+1;
        k = l;
        while((i<=m) && (j<=r))
        {
            if (feld[i] <= feld[j])
            {
                hFeld[k] = feld[i];
                i++;
            } else {
                hFeld[k] = feld[j];
                j++;
            }
            k++;
        }
        if (i > m)
        {
            for (int h=j; h<=r;h++)
            {
                hFeld[k+h-j] = feld[h];
            }
        } else         {
            for (int h=i; h<=m;h++)
            {
                hFeld[k+h-i] = feld[h];
            }
        }
        // Zurckkopieren
        for (int h = l; h<=r; h++)
            feld[h] = hFeld[h];

    }

    /**
     * Quicksort
     */
    public void quickSort(int links, int rechts)
    {
        int merkerLinks = links;    //Merker links
        int merkerRechts = rechts;  //Merker rechts
        int pivot = getFeld((links+rechts+1)/2); //Element in der Mitte ist Pivot
        do {
    
            while (getFeld(merkerLinks) < pivot)
                merkerLinks++;
            while (getFeld(merkerRechts) > pivot)
                merkerRechts--;
            
            if (merkerLinks <= merkerRechts)
            {
                tausche(merkerRechts,merkerLinks);
                merkerLinks++;
                merkerRechts--;
            }
        } while (merkerLinks <= merkerRechts);
        
        if (merkerRechts > links)
            quickSort(links, merkerRechts); //Sortieren des rechten Teils
        if (merkerLinks < rechts)
            quickSort(merkerLinks, rechts); //Sortieren des linken Teils
    }

}
